@@ -14,11 +14,12 @@ package MCE::Core::Input::Request;
 use strict;
 use warnings;
 
-our $VERSION = '1.703';
+our $VERSION = '1.843';
 
 ## Items below are folded into MCE.
 
-package MCE;
+package # hide from rpm
+   MCE;
 
 no warnings qw( threads recursion uninitialized );
 
@@ -48,26 +49,37 @@ sub _worker_request_chunk {
    my $_chunk_size  = $self->{chunk_size};
    my $_use_slurpio = $self->{use_slurpio};
    my $_RS          = $self->{RS} || $/;
-   my $_RS_FLG      = (!$_RS || $_RS ne $LF);
-   my $_I_FLG       = (!$/ || $/ ne $LF);
    my $_wuf         = $self->{_wuf};
 
-   my ($_dat_ex, $_dat_un);
+   my ($_dat_ex, $_dat_un, $_pid);
 
    if ($_lock_chn) {
-      $_dat_ex = sub { 1 until sysread(  $_DAT_LOCK->{_r_sock}, my $_b, 1 ) };
-      $_dat_un = sub { 1 until syswrite( $_DAT_LOCK->{_w_sock}, '0' ) };
+      $_pid = $INC{'threads.pm'} ? $$ .'.'. threads->tid() : $$;
+
+      # inlined for performance
+      $_dat_ex = sub {
+         MCE::Util::_sysread($_DAT_LOCK->{_r_sock}, my($b), 1), $_DAT_LOCK->{ $_pid } = 1
+            unless $_DAT_LOCK->{ $_pid };
+      };
+      $_dat_un = sub {
+         syswrite($_DAT_LOCK->{_w_sock}, '0'), $_DAT_LOCK->{ $_pid } = 0
+            if $_DAT_LOCK->{ $_pid };
+      };
    }
 
    my ($_chunk_id, $_len, $_output_tag);
    my ($_chop_len, $_chop_str, $_p);
 
    if ($_proc_type == REQUEST_ARRAY) {
-      $_output_tag = OUTPUT_A_ARY;
+      $_output_tag = OUTPUT_A_REF;
+      $_chop_len   = 0;
+   }
+   elsif ($_proc_type == REQUEST_HASH) {
+      $_output_tag = OUTPUT_H_REF;
       $_chop_len   = 0;
    }
    else {
-      $_output_tag = OUTPUT_S_GLB;
+      $_output_tag = OUTPUT_G_REF;
       if (length $_RS > 1 && substr($_RS, 0, 1) eq "\n") {
          $_chop_str = substr($_RS, 1);
          $_chop_len = length $_chop_str;
@@ -87,13 +99,14 @@ sub _worker_request_chunk {
    _WORKER_REQUEST_CHUNK__NEXT:
 
    while (1) {
-      undef $_ if (length > MAX_CHUNK_SIZE);
+      undef $_ if (length > MAX_GC_SIZE);
 
       $_ = '';
 
       ## Obtain the next chunk of data.
       {
-         local $\ = undef if (defined $\); local $/ = $LF if ($_I_FLG);
+         local $\ = undef if (defined $\);
+         local $/ = $LF   if ($/ ne $LF );
 
          $_dat_ex->() if $_lock_chn;
          print {$_DAT_W_SOCK} $_output_tag . $LF . $_chn . $LF;
@@ -128,6 +141,11 @@ sub _worker_request_chunk {
             $_wuf->($self, $_chunk_ref, $_chunk_id);
          }
       }
+      elsif ($_proc_type == REQUEST_HASH) {
+         my $_chunk_ref = { @{ $self->{thaw}($_) } }; undef $_;
+         $_ = $_chunk_ref;
+         $_wuf->($self, $_chunk_ref, $_chunk_id);
+      }
       else {
          if ($_use_slurpio) {
             if ($_chop_len && substr($_, -$_chop_len) eq $_chop_str) {
@@ -146,7 +164,7 @@ sub _worker_request_chunk {
             else {
                my @_recs;
                {
-                  local $/ = $_RS if ($_RS_FLG);
+                  local $/ = $_RS if ($/ ne $_RS);
                   _sync_buffer_to_array(\$_, \@_recs, $_chop_str);
                   undef $_;
                }
