@@ -14,16 +14,26 @@ package MCE::Core::Manager;
 use strict;
 use warnings;
 
-our $VERSION = '1.703';
+our $VERSION = '1.843';
 
+## no critic (BuiltinFunctions::ProhibitStringyEval)
 ## no critic (TestingAndDebugging::ProhibitNoStrict)
 
 ## Items below are folded into MCE.
 
-package MCE;
+package # hide from rpm
+   MCE;
 
 no warnings qw( threads recursion uninitialized );
 
+## The POSIX module has many symbols. Try not loading it simply
+## to have WNOHANG. The following covers most platforms.
+
+use constant {
+   _WNOHANG => ( $INC{'POSIX.pm'} )
+      ? &POSIX::WNOHANG : ( $^O eq 'solaris' ) ? 64 : 1
+};
+
 use bytes;
 
 ###############################################################################
@@ -78,41 +88,39 @@ sub _output_loop {
       $_has_user_tasks, $_sess_dir, $_task_id, $_user_error, $_user_output,
       $_input_size, $_offset_pos, $_single_dim, @_gather, $_cs_one_flag,
       $_exit_id, $_exit_pid, $_exit_status, $_exit_wid, $_len, $_sync_cnt,
-      $_BSB_W_SOCK, $_BSE_W_SOCK, $_DAT_R_SOCK, $_DAU_R_SOCK, $_MCE_STDERR,
-      $_I_FLG, $_O_FLG, $_I_SEP, $_O_SEP, $_RS, $_RS_FLG, $_MCE_STDOUT
+      $_BSB_W_SOCK, $_BSB_R_SOCK, $_DAT_R_SOCK, $_DAU_R_SOCK, $_MCE_STDERR,
+      $_I_FLG, $_O_FLG, $_I_SEP, $_O_SEP, $_RS, $_RS_FLG, $_MCE_STDOUT,
+      @_delay_wid, $_size_completed, $_win32_ipc
    );
 
    ## -------------------------------------------------------------------------
    ## Callback return.
 
-   my $_cb_ret_a = sub {                          # CBK return array
+   my $_cb_reply = sub {
+      local $\ = $_O_SEP if ($_O_FLG);
+      local $/ = $_I_SEP if ($_I_FLG);
 
-      my $_buf = $self->{freeze}($_[0]);
-         $_len = length $_buf; local $\ = undef if (defined $\);
+      no strict 'refs';
 
-      print {$_DAU_R_SOCK} $_len.$LF, $_buf;
+      if ( $_wa == WANTS_UNDEF ) {
+         $_cb->(@_);
+         return;
+      }
+      elsif ( $_wa == WANTS_ARRAY ) {
+         my @_ret = $_cb->(@_);
+         my $_buf = $self->{freeze}(\@_ret);
 
-      return;
-   };
+         return print {$_DAU_R_SOCK} length($_buf).'1'.$LF, $_buf;
+      }
 
-   my $_cb_ret_r = sub {                          # CBK return reference
+      my $_ret = $_cb->(@_);
 
-      my $_buf = $self->{freeze}($_[0]);
-         $_len = length $_buf; local $\ = undef if (defined $\);
+      return print {$_DAU_R_SOCK} length($_ret).'0'.$LF, $_ret
+         if ( !looks_like_number $_ret && !ref $_ret && defined $_ret );
 
-      print {$_DAU_R_SOCK} WANTS_REF.$LF . $_len.$LF, $_buf;
+      my $_buf = $self->{freeze}([ $_ret ]);
 
-      return;
-   };
-
-   my $_cb_ret_s = sub {                          # CBK return scalar
-
-      $_len = (defined $_[0]) ? length $_[0] : -1;
-      local $\ = undef if (defined $\);
-
-      print {$_DAU_R_SOCK} WANTS_SCALAR.$LF . $_len.$LF, $_[0];
-
-      return;
+      return print {$_DAU_R_SOCK} length($_buf).'1'.$LF, $_buf;
    };
 
    ## -------------------------------------------------------------------------
@@ -139,7 +147,9 @@ sub _output_loop {
 
          if ($_task_id == 0 && defined $_syn_flag && $_sync_cnt) {
             if ($_sync_cnt == $_total_running) {
-               for (1 .. $_total_running) { 1 until syswrite $_BSB_W_SOCK, $LF }
+               for my $_i (1 .. $_total_running) {
+                  syswrite($_BSB_W_SOCK, $LF);
+               }
                undef $_syn_flag;
             }
          }
@@ -169,7 +179,9 @@ sub _output_loop {
 
          if ($_task_id == 0 && defined $_syn_flag && $_sync_cnt) {
             if ($_sync_cnt == $_total_running) {
-               for (1 .. $_total_running) { 1 until syswrite $_BSB_W_SOCK, $LF }
+               for my $_i (1 .. $_total_running) {
+                  syswrite($_BSB_W_SOCK, $LF);
+               }
                undef $_syn_flag;
             }
          }
@@ -192,8 +204,6 @@ sub _output_loop {
             $self->{_wrk_status} = $_exit_status;
          }
 
-         print {$_DAU_R_SOCK} $LF;
-
          ## Reap child/thread. Note: Win32 uses negative PIDs.
 
          if ($_exit_pid =~ /^PID_(-?\d+)/) {
@@ -210,7 +220,7 @@ sub _output_loop {
             my $_tid = $1; my $_list = $self->{_tids};
             for my $i (0 .. @{ $_list }) {
                if ($_list->[$i] && $_list->[$i] == $_tid) {
-                  ${ $self->{_thrs}->[$i] }->join();
+                  $self->{_thrs}->[$i]->join();
                   $self->{_thrs}->[$i] = undef;
                   $self->{_tids}->[$i] = undef;
                   last;
@@ -258,7 +268,7 @@ sub _output_loop {
 
       ## ----------------------------------------------------------------------
 
-      OUTPUT_A_ARY.$LF => sub {                   # Array << Array
+      OUTPUT_A_REF.$LF => sub {                   # Input << Array ref
          my $_buf;
 
          if ($_offset_pos >= $_input_size || $_aborted) {
@@ -291,7 +301,7 @@ sub _output_loop {
          return;
       },
 
-      OUTPUT_S_GLB.$LF => sub {                   # Scalar << Glob FH
+      OUTPUT_G_REF.$LF => sub {                   # Input << Glob ref
          my $_buf = '';
 
          ## The logic below honors ('Ctrl/Z' in Windows, 'Ctrl/D' in Unix)
@@ -348,7 +358,37 @@ sub _output_loop {
          return;
       },
 
-      OUTPUT_U_ITR.$LF => sub {                   # User << Iterator
+      OUTPUT_H_REF.$LF => sub {                   # Input << Hash ref
+         my @_pairs;
+
+         if ($_offset_pos >= $_input_size || $_aborted) {
+            local $\ = undef if (defined $\);
+            print {$_DAU_R_SOCK} '0'.$LF;
+
+            return;
+         }
+
+         if ($_offset_pos + $_chunk_size - 1 < $_input_size) {
+            for my $_i ($_offset_pos .. $_offset_pos + $_chunk_size - 1) {
+               push @_pairs, each %{ $_input_data };
+            }
+         }
+         else {
+            for my $_i ($_offset_pos .. $_input_size - 1) {
+               push @_pairs, each %{ $_input_data };
+            }
+         }
+
+         my $_buf = $self->{freeze}(\@_pairs);
+
+         $_len = length $_buf; local $\ = undef if (defined $\);
+         print {$_DAU_R_SOCK} $_len.$LF . (++$_chunk_id).$LF, $_buf;
+         $_offset_pos += $_chunk_size;
+
+         return;
+      },
+
+      OUTPUT_I_REF.$LF => sub {                   # Input << Iter ref
          my $_buf;
 
          if ($_aborted) {
@@ -361,7 +401,7 @@ sub _output_loop {
          my @_ret_a = $_input_data->($_chunk_size);
 
          if (scalar @_ret_a > 1 || ref $_ret_a[0]) {
-            $_buf = $self->{freeze}( [ @_ret_a ] );
+            $_buf = $self->{freeze}([ @_ret_a ]);
             $_len = length $_buf; local $\ = undef if (defined $\);
             print {$_DAU_R_SOCK} $_len.'1'.$LF . (++$_chunk_id).$LF, $_buf;
 
@@ -384,92 +424,40 @@ sub _output_loop {
       ## ----------------------------------------------------------------------
 
       OUTPUT_A_CBK.$LF => sub {                   # Callback w/ multiple args
-         my ($_buf, $_data_ref);
-
          chomp($_wa  = <$_DAU_R_SOCK>),
          chomp($_cb  = <$_DAU_R_SOCK>),
          chomp($_len = <$_DAU_R_SOCK>);
 
-         read $_DAU_R_SOCK, $_buf, $_len;
-         $_data_ref = $self->{thaw}($_buf); undef $_buf;
+         read $_DAU_R_SOCK, my($_buf), $_len;
+         my $_data_ref = $self->{thaw}($_buf); undef $_buf;
 
-         local $\ = $_O_SEP if ($_O_FLG); local $/ = $_I_SEP if ($_I_FLG);
-         no strict 'refs';
-
-         if ($_wa == WANTS_UNDEF) {
-            $_cb->(@{ $_data_ref });
-         }
-         elsif ($_wa == WANTS_ARRAY) {
-            my @_ret_a = $_cb->(@{ $_data_ref });
-            $_cb_ret_a->(\@_ret_a);
-         }
-         else {
-            my  $_ret_s = $_cb->(@{ $_data_ref });
-            ref $_ret_s ? $_cb_ret_r->($_ret_s) : $_cb_ret_s->($_ret_s);
-         }
-
-         return;
+         return $_cb_reply->(@{ $_data_ref });
       },
 
       OUTPUT_S_CBK.$LF => sub {                   # Callback w/ 1 scalar arg
-         my $_buf;
-
          chomp($_wa  = <$_DAU_R_SOCK>),
          chomp($_cb  = <$_DAU_R_SOCK>),
          chomp($_len = <$_DAU_R_SOCK>);
 
-         read $_DAU_R_SOCK, $_buf, $_len;
+         read $_DAU_R_SOCK, my($_buf), $_len;
 
-         local $\ = $_O_SEP if ($_O_FLG); local $/ = $_I_SEP if ($_I_FLG);
-         no strict 'refs';
-
-         if ($_wa == WANTS_UNDEF) {
-            $_cb->($_buf);
-         }
-         elsif ($_wa == WANTS_ARRAY) {
-            my @_ret_a = $_cb->($_buf);
-            $_cb_ret_a->(\@_ret_a);
-         }
-         else {
-            my  $_ret_s = $_cb->($_buf);
-            ref $_ret_s ? $_cb_ret_r->($_ret_s) : $_cb_ret_s->($_ret_s);
-         }
-
-         return;
+         return $_cb_reply->($_buf);
       },
 
       OUTPUT_N_CBK.$LF => sub {                   # Callback w/ no args
-
          chomp($_wa = <$_DAU_R_SOCK>),
          chomp($_cb = <$_DAU_R_SOCK>);
 
-         local $\ = $_O_SEP if ($_O_FLG); local $/ = $_I_SEP if ($_I_FLG);
-         no strict 'refs';
-
-         if ($_wa == WANTS_UNDEF) {
-            $_cb->();
-         }
-         elsif ($_wa == WANTS_ARRAY) {
-            my @_ret_a = $_cb->();
-            $_cb_ret_a->(\@_ret_a);
-         }
-         else {
-            my  $_ret_s = $_cb->();
-            ref $_ret_s ? $_cb_ret_r->($_ret_s) : $_cb_ret_s->($_ret_s);
-         }
-
-         return;
+         return $_cb_reply->();
       },
 
       ## ----------------------------------------------------------------------
 
       OUTPUT_A_GTR.$LF => sub {                   # Gather array/ref
-         my $_buf;
-
          chomp($_task_id = <$_DAU_R_SOCK>),
          chomp($_len     = <$_DAU_R_SOCK>);
 
-         read $_DAU_R_SOCK, $_buf, $_len;
+         read $_DAU_R_SOCK, my($_buf), $_len;
 
          if ($_is_c_ref[$_task_id]) {
             local $_ = $self->{thaw}($_buf);
@@ -520,10 +508,8 @@ sub _output_loop {
       ## ----------------------------------------------------------------------
 
       OUTPUT_O_SND.$LF => sub {                   # Send >> STDOUT
-         my $_buf;
-
          chomp($_len = <$_DAU_R_SOCK>);
-         read $_DAU_R_SOCK, $_buf, $_len;
+         read $_DAU_R_SOCK, my($_buf), $_len;
 
          if (defined $_user_output) {
             $_user_output->($_buf);
@@ -535,10 +521,8 @@ sub _output_loop {
       },
 
       OUTPUT_E_SND.$LF => sub {                   # Send >> STDERR
-         my $_buf;
-
          chomp($_len = <$_DAU_R_SOCK>);
-         read $_DAU_R_SOCK, $_buf, $_len;
+         read $_DAU_R_SOCK, my($_buf), $_len;
 
          if (defined $_user_error) {
             $_user_error->($_buf);
@@ -563,9 +547,9 @@ sub _output_loop {
 
             binmode $_sendto_fhs{$_file};
 
-            ## Select new FH, turn on autoflush, restore the old FH.
             if ($_flush_file) {
-               local $|; select((select($_sendto_fhs{$_file}), $| = 1)[0]);
+               local $!;
+               $_sendto_fhs{$_file}->autoflush(1);
             }
          }
 
@@ -584,17 +568,15 @@ sub _output_loop {
          read $_DAU_R_SOCK, $_buf, $_len;
 
          unless (exists $_sendto_fhs{$_fd}) {
-            require IO::Handle unless (defined $IO::Handle::VERSION);
-
             $_sendto_fhs{$_fd} = IO::Handle->new();
             $_sendto_fhs{$_fd}->fdopen($_fd, 'w')
                or _croak "Cannot open file descriptor ($_fd): $!";
 
             binmode $_sendto_fhs{$_fd};
 
-            ## Select new FH, turn on autoflush, restore the old FH.
             if ($_flush_file) {
-               local $|; select((select($_sendto_fhs{$_fd}), $| = 1)[0]);
+               local $!;
+               $_sendto_fhs{$_fd}->autoflush(1);
             }
          }
 
@@ -607,10 +589,8 @@ sub _output_loop {
       ## ----------------------------------------------------------------------
 
       OUTPUT_B_SYN.$LF => sub {                   # Barrier sync - begin
-
          if (!defined $_sync_cnt || $_sync_cnt == 0) {
-            $_syn_flag = 1;
-            $_sync_cnt = 0;
+            $_syn_flag = 1, $_sync_cnt = 0;
          }
 
          my $_total_running = ($_has_user_tasks)
@@ -618,7 +598,9 @@ sub _output_loop {
             : $self->{_total_running};
 
          if (++$_sync_cnt == $_total_running) {
-            for (1 .. $_total_running) { 1 until syswrite $_BSB_W_SOCK, $LF; }
+            for my $_i (1 .. $_total_running) {
+               syswrite($_BSB_W_SOCK, $LF);
+            }
             undef $_syn_flag;
          }
 
@@ -626,13 +608,86 @@ sub _output_loop {
       },
 
       OUTPUT_E_SYN.$LF => sub {                   # Barrier sync - end
-
          if (--$_sync_cnt == 0) {
             my $_total_running = ($_has_user_tasks)
                ? $self->{_task}->[0]->{_total_running}
                : $self->{_total_running};
 
-            for (1 .. $_total_running) { 1 until syswrite $_BSE_W_SOCK, $LF; }
+            for my $_i (1 .. $_total_running) {
+               syswrite($_BSB_R_SOCK, $LF);
+            }
+         }
+
+         return;
+      },
+
+      OUTPUT_S_IPC.$LF => sub {                   # Change to win32 IPC
+         syswrite($_DAT_R_SOCK, $LF);
+
+         $_win32_ipc = 1, goto _LOOP unless $_win32_ipc;
+
+         return;
+      },
+
+      OUTPUT_P_NFY.$LF => sub {                   # Progress notification
+         chomp($_len = <$_DAU_R_SOCK>);
+
+         $self->{progress}->( $_size_completed += $_len );
+
+         return;
+      },
+
+      OUTPUT_S_DIR.$LF => sub {                   # Make/get sess_dir
+         print {$_DAU_R_SOCK} $self->sess_dir().$LF;
+
+         return;
+      },
+
+      OUTPUT_T_DIR.$LF => sub {                   # Make/get tmp_dir
+         print {$_DAU_R_SOCK} $self->tmp_dir().$LF;
+
+         return;
+      },
+
+      OUTPUT_I_DLY.$LF => sub {                   # Interval delay
+         my $_tasks = $_has_user_tasks ? $self->{user_tasks} : undef;
+
+         chomp($_task_id = <$_DAU_R_SOCK>);
+
+         my $_interval = ($_tasks && exists $_tasks->[$_task_id]{interval})
+            ? $_tasks->[$_task_id]{interval}
+            : $self->{interval};
+
+         if ( $_interval ) {
+            my $_max_workers = ($_tasks)
+               ? $_tasks->[$_task_id]{max_workers}
+               : $self->{max_workers};
+
+            $_delay_wid[$_task_id] = 1
+               if (++$_delay_wid[$_task_id] > $_max_workers);
+
+            my $_nodes  = $_interval->{max_nodes};
+            my $_id     = $_interval->{node_id};
+            my $_delay  = $_interval->{delay} * $_nodes;
+
+            my $_app_tb = $_delay * $_max_workers;
+            my $_app_st = $_interval->{_time} + ($_delay / $_nodes * $_id);
+            my $_wrk_st = ($_delay_wid[$_task_id] - 1) * $_delay + $_app_st;
+
+            $_delay = $_wrk_st - time;
+
+            if ($_delay < 0.0 && $_app_tb) {
+               my $_count = int($_delay * -1 / $_app_tb + 0.5) + 1;
+               $_delay += $_app_tb * $_count;
+               $_interval->{_time} = time if ($_count > 2e9);
+            }
+
+            ($_delay > 0.0)
+               ? print {$_DAU_R_SOCK} $_delay.$LF
+               : print {$_DAU_R_SOCK} '0'.$LF;
+         }
+         else {
+            print {$_DAU_R_SOCK} '0'.$LF;
          }
 
          return;
@@ -642,13 +697,13 @@ sub _output_loop {
 
    ## -------------------------------------------------------------------------
 
-   local ($!, $_);
+   local ($!, $?, $_);
 
+   $_aborted = $_chunk_id = $_eof_flag = $_size_completed = 0;
    $_has_user_tasks = (defined $self->{user_tasks}) ? 1 : 0;
    $_cs_one_flag = ($self->{chunk_size} == 1) ? 1 : 0;
-   $_aborted = $_chunk_id = $_eof_flag = 0;
 
-   $_max_retries  = $self->{max_retries};
+   $_max_retries  = $self->{max_retries} || 0;
    $_on_post_exit = $self->{on_post_exit};
    $_on_post_run  = $self->{on_post_run};
    $_chunk_size   = $self->{chunk_size};
@@ -661,10 +716,10 @@ sub _output_loop {
    if ($_max_retries && !$_on_post_exit) {
       $_on_post_exit = sub {
          my ($self, $_e, $_retry_cnt) = @_;
-         my ($_cnt, $_msg) = ($_retry_cnt + 1, "Error: Chunk $_e->{id} failed");
+         my ($_cnt, $_msg) = ($_retry_cnt + 1, "Error: chunk $_e->{id} failed");
 
          ($_retry_cnt < $_max_retries)
-            ? print {*STDERR} "$_msg, retrying chunk attempt #${_cnt}\n"
+            ? print {*STDERR} "$_msg, retrying chunk attempt # ${_cnt}\n"
             : print {*STDERR} "$_msg\n";
 
          $self->restart_worker;
@@ -703,7 +758,12 @@ sub _output_loop {
    if (defined $_input_data && ref $_input_data eq 'ARRAY') {
       $_input_size = @{ $_input_data };
       $_offset_pos = 0;
-   } else {
+   }
+   elsif (defined $_input_data && ref $_input_data eq 'HASH') {
+      $_input_size = scalar( keys %{ $_input_data } );
+      $_offset_pos = 0;
+   }
+   else {
       $_input_size = $_offset_pos = 0;
    }
 
@@ -715,7 +775,7 @@ sub _output_loop {
       binmode $_MCE_STDOUT;
    }
    else {
-      open $_MCE_STDOUT, '>&=STDOUT';
+      $_MCE_STDOUT = \*STDOUT;
       binmode $_MCE_STDOUT;
    }
 
@@ -725,30 +785,33 @@ sub _output_loop {
       binmode $_MCE_STDERR;
    }
    else {
-      open $_MCE_STDERR, '>&=STDERR';
+      $_MCE_STDERR = \*STDERR;
       binmode $_MCE_STDERR;
    }
 
-   ## Make MCE_STDOUT the default handle.
-   ## Flush STDERR/STDOUT handles if requested.
+   ## Autoflush STDERR-STDOUT handles if requested.
 
-   my $_old_hndl = select $_MCE_STDOUT;
-
-   if ($self->{flush_stdout}) {
-      local $|; select((select($_MCE_STDOUT), $| = 1)[0]);
-   }
-   if ($self->{flush_stderr}) {
-      local $|; select((select($_MCE_STDERR), $| = 1)[0]);
+   {
+      local $!;
+      $_MCE_STDERR->autoflush(1) if $self->{flush_stderr};
+      $_MCE_STDOUT->autoflush(1) if $self->{flush_stdout};
    }
 
    ## -------------------------------------------------------------------------
 
    ## Output event loop.
 
-   my $_func; my $_channels = $self->{_dat_r_sock};
+   my $_channels = $self->{_dat_r_sock};
+   my $_func;
+
+   $_win32_ipc = (
+      $ENV{'PERL_MCE_IPC'} eq 'win32' ||
+      $INC{'MCE/Child.pm'} ||
+      $INC{'MCE/Hobo.pm'}
+   );
 
    $_BSB_W_SOCK = $self->{_bsb_w_sock};
-   $_BSE_W_SOCK = $self->{_bse_w_sock};
+   $_BSB_R_SOCK = $self->{_bsb_r_sock};
    $_DAT_R_SOCK = $self->{_dat_r_sock}->[0];
 
    $_RS     = $self->{RS} || $/;
@@ -768,7 +831,9 @@ sub _output_loop {
    ## Wait on requests *with* timeout capability. Exit loop when all workers
    ## have completed processing or exited prematurely.
 
-   if ($self->{loop_timeout} && $self->{_pids} && $^O ne 'MSWin32') {
+   _LOOP:
+
+   if ($self->{loop_timeout} && @{ $self->{_tids} } == 0 && $^O ne 'MSWin32') {
       my ($_list, $_timeout) = ($self->{_pids}, $self->{loop_timeout});
       my ($_DAT_W_SOCK, $_pid) = ($self->{_dat_w_sock}->[0]);
 
@@ -778,7 +843,7 @@ sub _output_loop {
          alarm 0;
          for my $i (0 .. @{ $_list }) {
             if ($_pid = $_list->[$i]) {
-               if (waitpid($_pid, 1)) {
+               if (waitpid($_pid, _WNOHANG)) {
                   $self->{_total_exited}  += 1;
                   $self->{_total_running} -= 1;
                   $self->{_total_workers} -= 1;
@@ -803,12 +868,27 @@ sub _output_loop {
       }
    }
 
-   ## Otherwise, wait on requests *without* timeout capability.
-   ## Exit loop when all workers have completed processing.
+   ## Wait on requests *without* timeout capability.
 
+   elsif ($^O eq 'MSWin32') {
+      MCE::Util::_nonblocking($_DAT_R_SOCK, 1) if $_win32_ipc;
+
+      while ($self->{_total_running}) {
+         MCE::Util::_sysread2($_DAT_R_SOCK, $_func, 8);
+         last() unless length($_func) == 8;
+         $_DAU_R_SOCK = $_channels->[ substr($_func, -2, 2, '') ];
+
+         if (exists $_core_output_function{$_func}) {
+            $_core_output_function{$_func}();
+         } elsif (exists $_plugin_function->{$_func}) {
+            $_plugin_function->{$_func}();
+         }
+      }
+   }
    else {
-      while ( $self->{_total_running} ) {
+      while ($self->{_total_running}) {
          $_func = <$_DAT_R_SOCK>;
+         last() unless length($_func) == 6;
          $_DAU_R_SOCK = $_channels->[ <$_DAT_R_SOCK> ];
 
          if (exists $_core_output_function{$_func}) {
@@ -837,12 +917,12 @@ sub _output_loop {
       delete $_sendto_fhs{$_p};
    }
 
-   ## Restore the default handle. Close MCE STDOUT/STDERR handles.
+   ## Close MCE STDOUT/STDERR handles.
 
-   select $_old_hndl;
-
-   close $_MCE_STDOUT; undef $_MCE_STDOUT;
-   close $_MCE_STDERR; undef $_MCE_STDERR;
+   eval q{
+      close $_MCE_STDOUT if (fileno $_MCE_STDOUT > 2);
+      close $_MCE_STDERR if (fileno $_MCE_STDERR > 2);
+   };
 
    return;
 }
