@@ -11,7 +11,7 @@ use warnings;
 
 no warnings qw( threads recursion uninitialized );
 
-our $VERSION = '1.703';
+our $VERSION = '1.843';
 
 ## no critic (Subroutines::ProhibitSubroutinePrototypes)
 ## no critic (TestingAndDebugging::ProhibitNoStrict)
@@ -25,11 +25,9 @@ use MCE::Relay;
 ##
 ###############################################################################
 
-my $_imported;
-
 sub import {
 
-   my $_class = shift; return if ($_imported++);
+   shift;
 
    my $_g_flg = 0; my $_m_flg = 0; my $_w_flg = 0;
    my $_flag = sub { 1 }; my $_package = caller;
@@ -77,7 +75,6 @@ sub mce_status      ( ) { return $MCE::MCE->status(); }
 
 ## Callable by the worker process only.
 
-sub mce_do          (@) { return $MCE::MCE->do(@_); }
 sub mce_exit        (@) { return $MCE::MCE->exit(@_); }
 sub mce_gather      (@) { return $MCE::MCE->gather(@_); }
 sub mce_last        ( ) { return $MCE::MCE->last(); }
@@ -91,6 +88,7 @@ sub mce_yield       ( ) { return $MCE::MCE->yield(); }
 ## Callable by both the manager and worker processes.
 
 sub mce_abort       ( ) { return $MCE::MCE->abort(); }
+sub mce_do          (@) { return $MCE::MCE->do(@_); }
 sub mce_freeze      (@) { return $MCE::MCE->{freeze}(@_); }
 sub mce_print     (;*@) { return $MCE::MCE->print(@_); }
 sub mce_printf    (;*@) { return $MCE::MCE->printf(@_); }
@@ -149,7 +147,6 @@ sub _export_subs {
    ## Callable by the worker process only.
 
    if ($_w_flg) {
-      *{ $_package . '::mce_do'          } = \&mce_do;
       *{ $_package . '::mce_exit'        } = \&mce_exit;
       *{ $_package . '::mce_gather'      } = \&mce_gather;
       *{ $_package . '::mce_last'        } = \&mce_last;
@@ -165,6 +162,7 @@ sub _export_subs {
 
    if ($_m_flg || $_w_flg) {
       *{ $_package . '::mce_abort'       } = \&mce_abort;
+      *{ $_package . '::mce_do'          } = \&mce_do;
       *{ $_package . '::mce_freeze'      } = \&mce_freeze;
       *{ $_package . '::mce_print'       } = \&mce_print;
       *{ $_package . '::mce_printf'      } = \&mce_printf;
@@ -206,72 +204,72 @@ MCE::Subs - Exports functions mapped directly to MCE methods
 
 =head1 VERSION
 
-This document describes MCE::Subs version 1.703
+This document describes MCE::Subs version 1.843
 
 =head1 SYNOPSIS
 
-   use MCE::Subs;  ## Exports manager and worker functions only
-                   ## Getter functions are not exported by default
+ use MCE::Subs;  ## Exports manager and worker functions only
+                 ## Getter functions are not exported by default
 
-   use MCE::Subs qw( :getter  );  ## All, including getter functions
-   use MCE::Subs qw( :manager );  ## Exports manager functions only
-   use MCE::Subs qw( :worker  );  ## Exports worker functions only
+ use MCE::Subs qw( :getter  );  ## All, including getter functions
+ use MCE::Subs qw( :manager );  ## Exports manager functions only
+ use MCE::Subs qw( :worker  );  ## Exports worker functions only
 
-   use MCE::Subs qw( :getter :worker );  ## Excludes manager functions
+ use MCE::Subs qw( :getter :worker );  ## Excludes manager functions
 
 =head1 DESCRIPTION
 
 This module exports functions mapped to MCE methods. All exported functions
 are prototyped, therefore allowing one to call them without using parentheses.
 
-   use MCE::Subs qw( :worker );
+ use MCE::Subs qw( :worker );
 
-   sub user_func {
-      my $wid = MCE->wid;
+ sub user_func {
+    my $wid = MCE->wid;
 
-      mce_say "A: $wid";
-      mce_sync;
+    mce_say "A: $wid";
+    mce_sync;
 
-      mce_say "B: $wid";
-      mce_sync;
+    mce_say "B: $wid";
+    mce_sync;
 
-      mce_say "C: $wid";
-      mce_sync;
+    mce_say "C: $wid";
+    mce_sync;
 
-      return;
-   }
+    return;
+ }
 
-   MCE->new(
-      max_workers => 24, user_func => \&user_func
-   );
+ MCE->new(
+    max_workers => 24, user_func => \&user_func
+ );
 
-   mce_run 0 for (1..100);   ## 0 means do not shutdown after running
+ mce_run 0 for (1..100);   ## 0 means do not shutdown after running
 
 For the next example, we only want the worker functions to be exported due
 to using MCE::Map, which takes care of creating a MCE instance and running.
 
-   use MCE::Map;
-   use MCE::Subs qw( :worker );
+ use MCE::Map;
+ use MCE::Subs qw( :worker );
 
-   ## The following serializes output to STDOUT and gathers $_ to @a.
-   ## mce_say displays $_ when called without arguments.
+ ## The following serializes output to STDOUT and gathers $_ to @a.
+ ## mce_say displays $_ when called without arguments.
 
-   my @a = mce_map { mce_say; $_ } 1 .. 100;
+ my @a = mce_map { mce_say; $_ } 1 .. 100;
 
-   print scalar @a, "\n";
+ print scalar @a, "\n";
 
 Unlike the native Perl functions, printf, print, and say methods require the
 comma after the glob reference or file handle.
 
-   MCE->printf(\*STDERR, "%s\n", $error_msg);
-   MCE->print(\*STDERR, $error_msg, "\n");
-   MCE->say(\*STDERR, $error_msg);
-   MCE->say($fh, $error_msg);
+ MCE->printf(\*STDERR, "%s\n", $error_msg);
+ MCE->print(\*STDERR, $error_msg, "\n");
+ MCE->say(\*STDERR, $error_msg);
+ MCE->say($fh, $error_msg);
 
-   mce_printf \*STDERR, "%s\n", $error_msg;
-   mce_print \*STDERR, $error_msg, "\n";
-   mce_say \*STDERR, $error_msg;
-   mce_say $fh, $error_msg;
+ mce_printf \*STDERR, "%s\n", $error_msg;
+ mce_print \*STDERR, $error_msg, "\n";
+ mce_say \*STDERR, $error_msg;
+ mce_say $fh, $error_msg;
 
 =head1 FUNCTIONS for the MANAGER PROCESS via ( :manager )
 
@@ -281,6 +279,8 @@ MCE methods are described in L<MCE::Core>.
 
 =item * mce_abort
 
+=item * mce_do
+
 =item * mce_forchunk
 
 =item * mce_foreach
